#
# BigBrotherBot(B3) (www.bigbrotherbot.com)
# Copyright (C) 2005 Michael "ThorN" Thornton
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

__author__  = 'ThorN, xlr8or'
__version__ = '1.2.0'

import re

#--------------------------------------------------------------------------------------------------
def splitDSN(url):
  m = re.match(r'^(?:(?P<protocol>[a-z]+)://)?(?:(?P<user>[^@:]+)(?::(?P<password>[^@]+))?@)?(?P<host>[^/:]+)?(?::(?P<port>\d+))?(?P<path>.*)', url)
  if not m:
    return None

  g = m.groupdict()
  if g['port']:
    g['port'] = int(g['port'])

  if not g['protocol']:
    g['protocol'] = 'file'
  if g['protocol'] == 'file':
    if g['host'] and g['path']:
      g['path'] = '%s%s' % (g['host'], g['path'])
      g['host'] = None
    elif g['host']:
      g['path'] = g['host']
      g['host'] = None
  elif g['protocol'] == 'exec':
    if g['host'] and g['path']:
      g['path'] = '%s/%s' % (g['host'], g['path'])
      g['host'] = None
    elif g['host']:
      g['path'] = g['host']
      g['host'] = None

  return g

#--------------------------------------------------------------------------------------------------
def minutes2int(mins):
  if re.match('^[0-9.]+$', mins):
    return round(float(mins), 2)
  else:
    return 0

#--------------------------------------------------------------------------------------------------
def time2minutes(timeStr):
  if not timeStr:
    return 0
  elif type(timeStr) is int:
    return timeStr

  timeStr = str(timeStr)
  if not timeStr:
    return 0
  elif timeStr[-1:] == 'h':
    return minutes2int(timeStr[:-1]) * 60
  elif timeStr[-1:] == 'm':
    return minutes2int(timeStr[:-1])
  elif timeStr[-1:] == 's':
    return minutes2int(timeStr[:-1]) / 60
  elif timeStr[-1:] == 'd':
    return minutes2int(timeStr[:-1]) * 60 * 24
  elif timeStr[-1:] == 'w':
    return minutes2int(timeStr[:-1]) * 60 * 24 * 7
  else:
    return minutes2int(timeStr)

#--------------------------------------------------------------------------------------------------
def minutesStr(timeStr):
  mins = time2minutes(timeStr)

  s = ''
  if mins < 1:
    num = round(mins * 60, 2)
    s = '%s second' % num
  elif mins < 60:
    num = round(mins, 2)
    s = '%s minute' % num
  elif mins < 1440:
    num = round(mins / 60, 2)
    s = '%s hour' % num
  elif mins < 10080:
    num = round((mins / 60) / 24, 2)
    s = '%s day' % num
  elif mins < 525600:
    num = round(((mins / 60) / 24) / 7, 2)
    s = '%s week' % num
  else:
    num = round(((mins / 60) / 24) / 365, 2)
    s = '%s year' % num

  if num != 1.0:
    s += 's'

  return s

def vars2printf(inputStr):
  return re.sub(r'\$([a-zA-Z]+)', r'%(\1)s', inputStr)
  
#--------------------------------------------------------------------------------------------------
def levenshteinDistance(a,b):
    c = {}
    n = len(a); m = len(b)

    for i in range(0,n+1):
        c[i,0] = i
    for j in range(0,m+1):
        c[0,j] = j

    for i in range(1,n+1):
        for j in range(1,m+1):
            x = c[i-1,j]+1
            y = c[i,j-1]+1
            if a[i-1] == b[j-1]:
                z = c[i-1,j-1]
            else:
                z = c[i-1,j-1]+1
            c[i,j] = min(x,y,z)
    return c[n,m]

def fuzzyGuidMatch(a, b):
    a = a.upper()
    b = b.upper()

    if a == b:
        return True
    
    # put the longest first
    if len(b) > len(a):
        a, b = b, a

    if len(a) == 32 and len(b) == 31:
        # Looks like a truncated id, check using levenshtein
        # Use levenshteinDistance to find GUIDs off by 1 char, as caused by a bug in COD Punkbuster
        distance = levenshteinDistance(a, b)
        if distance <= 1:
            return True
    
    return False

if __name__ == '__main__':
    print splitDSN('sqlite://c|/mydatabase/test.db')

    compare = [
        ('098f6bcd4621d373cade4e832627b4f6', '098f6bcd4621d373cade4e832627b4f6'),
        ('098f6bcd4621d373cade4e832627b4f6', '098f6bcd4621d373cade4e832627b4f'),
        ('098f6bcd4621d373cade4e832627bf6', '098f6bcd4621d373cade4e832627b4f6'),
        ('098f6bcd4621d373cade4e832627b4f6', '098f6bcd4621d373cde4e832627b4f6'),
        ('098f6bcd4621d373cade4e832627b4f6', '098f6bcd46d373cade4e832627b4f6'),
        ('098f6bcd4621d373cade4832627b4f6', '098f6bcd4621d73cade4e832627b4f6'),
        ('098F6BCD4621D373CADE4E832627B4F6', '098f6bcd4621d373cade4e832627b4f6'),
    ]


    for a,b in compare:
        print '%s <> %s = %s' % (a, b, fuzzyGuidMatch(a,b))

#--------------------------------------------------------------------------------------------------
def sanitizeMe(s):
  sanitized = re.sub(r'[\x00-\x1F]|[\x7F-\xff]', '?', str(s))
  return sanitized
